# File that contains VW-specific class-extensions to SketchUp API classes.
# Author: K. Di Filippo

# Path
PLUGIN_PATH= 'Virtualwind/'

# Global requires
require 'sketchup.rb'
Sketchup.load PLUGIN_PATH + 'vwGlobalConstants'


# Extend the Entity class
class Sketchup::Entity
  
  # Method that indicates whether this Entity is complex.
  # An Entity is considered "complex" if it is a Group or Component.
  # Returns:
  #   - true if this Entity is complex; false otherwise.
  def is_complex_entity?
    
    is_complex = (self.kind_of? Sketchup::Group) || (self.kind_of? Sketchup::ComponentInstance) || (self.kind_of? Sketchup::ComponentDefinition) 
    return is_complex
    
  end
  
end


# Extend the Face class
class Sketchup::Face
  
  # Method that calls hidden= on itself and all connected Edges.
  # Params:
  #   - new_hidden_value: true to hide this Face and all connected
  #      Edges; false otherwise.
  def connected_hidden= new_hidden_value
    
    # Hide this Face.
    self.hidden=new_hidden_value
    
    # Hide all Edges connected to this Face.
    self.faces.each{|edge| edge.hidden=new_hidden_value}
    
  end
  
  
  # Method that calls visible= on itself and all connected  Edges.
  # Params:
  #   - new_visible_value: true to show this Face and all connected
  #      Edges; false otherwise.
  def connected_visible= new_visible_value
    
    # Show this Face.
    self.visible=new_visible_value
    
    # Show all Edges connected to this Face.
    self.edges.each{|edge| edge.visible=new_visible_value}
    
  end
  
end


# Extend the Edge class
class Sketchup::Edge
  
  # Method that calls hidden= on itself and all connected faces.
  # Params:
  #   - new_hidden_value: true to hide this Edge and all connected
  #     Faces; false otherwise.
  def connected_hidden= new_hidden_value
    
    # Hide this Edge.
    self.hidden=new_hidden_value
    
    # Hide all Faces connected to this Edge.
    self.faces.each{|face| face.hidden=new_hidden_value}
    
  end
  
  
  # Method that calls visible= on itself and all connected faces.
  # Params:
  #   - new_visible_value: true to show this Edge and all connected
  #     Faces; false otherwise.
  def connected_visible= new_visible_value
    
    # Show this Edge.
    self.visible=new_visible_value
    
    # Show all Faces connected to this Edge.
    self.faces.each{|face| face.visible=new_visible_value}
    
  end
  
  # Method that checks whether this Edge is a valid "solid Edge" -- that
  # is to say, whether this Edge has exactly two connected Faces.
  # Returns:
  #   - true if this Edge has exactly two connected Faces; 
  #     false otherwise.
  def valid_solid_edge?
    
    valid = (self.faces.length == 2)
    return valid
    
  end
  
  
end


# Extend the Group class
###
class Sketchup::Group
    # Sometimes 'group.entities.parent' refers to the wrong definition. 
    # This method checks for this error & finds the correct definition.
    if not Sketchup::Group.method_defined?(:definition)
       def definition()
          if self.entities.parent.instances.include?(self)
             return self.entities.parent
          else
             Sketchup.active_model.definitions.each { |definition|
                return definition if definition.instances.include?(self)
             }
          end
          return nil # Should not happen.
       end
    end#if
end#class Group
###


# Declare the file is loaded
file_loaded('vwSketchupExtensions')
